@@ -17,17 +17,21 @@ class WaitWaiter(Waiter):
         super(WaitWaiter, self).__init__(executor)
 
     def run_query(self, query, database):
-        while database.check_status() == DatabaseStatus.RUNNING.value:
+        if database.check_status() == DatabaseStatus.RUNNING.value:
+            database.is_down.acquire()
             try:
                 self._executor.run_query(query, database)
-                break
+                database.is_down.release()
             except django.db.utils.OperationalError:
+                database.is_up.clear()
                 database.change_status(DatabaseStatus.DOWN.value)
                 self.wait_for_connection(query, database)
+        else:
+            self.wait_for_connection(query, database)
 
     def wait_for_connection(self, query, database):
-        while database.check_status() == DatabaseStatus.DOWN.value:
-            pass
+        database.is_up.wait()
+        self.run_query(query,database)
 
 
 class DontWaitWaiter(Waiter):
@@ -36,5 +40,6 @@ class DontWaitWaiter(Waiter):
             try:
                 self._executor.run_query(query, database)
             except django.db.utils.OperationalError:
+                database.is_up.clear()
                 database.change_status(DatabaseStatus.DOWN.value)
                 raise django.db.utils.OperationalError
